package overflow_gateway_websocket

type (
	// OnPreRequestFunc is callback function
	OnPreRequestFunc  func()
	OnPostRequestFunc func()
	OnPreNotifyFunc   func()
	OnPrePushFunc     func()
)

type Server interface {
	OnPreRequest(cb OnPreRequestFunc)
	OnPostRequest(cb OnPostRequestFunc)
	OnPreNotify(cb OnPreNotifyFunc)
	OnPrePush(cb OnPrePushFunc)
	ListenAndServe()
}

type server struct {
	onPreRequestListeners  []OnPreRequestFunc
	onPostRequestListeners []OnPostRequestFunc
	onPreNotifyListeners   []OnPreNotifyFunc
	onPrePushListeners     []OnPrePushFunc
}

func NewServer() Server {
	s := &server{
		onPreRequestListeners:  make([]OnPreRequestFunc, 0),
		onPostRequestListeners: make([]OnPostRequestFunc, 0),
		onPreNotifyListeners:   make([]OnPreNotifyFunc, 0),
		onPrePushListeners:     make([]OnPrePushFunc, 0),
	}
	return s
}

func (s *server) OnPreRequest(cb OnPreRequestFunc) {
	s.onPreRequestListeners = append(s.onPreRequestListeners, cb)
}

func (s *server) OnPostRequest(cb OnPostRequestFunc) {
	s.onPostRequestListeners = append(s.onPostRequestListeners, cb)
}

func (s *server) OnPreNotify(cb OnPreNotifyFunc) {
	s.onPreNotifyListeners = append(s.onPreNotifyListeners, cb)
}

func (s *server) OnPrePush(cb OnPrePushFunc) {
	s.onPrePushListeners = append(s.onPrePushListeners, cb)
}

func (s *server) ListenAndServe() {

}
